#import "StupidModelSwarm.h"
#import "HabitatCell.h"


#import "StupidBug.h"


@implementation StupidBug

+ create: aZone
{
   StupidBug* stupidBug = [super create: aZone];

   stupidBug->stupidModelSwarm = nil;
   stupidBug->myBugWorld = nil;
   stupidBug->myCellWorld = nil;
   stupidBug->mySize = -1;

   return stupidBug;
}


////////////////////////////////
//
// setStupidModelSwarm
//
////////////////////////////////
- setStupidModelSwarm: aStupidModelSwarm
{
    stupidModelSwarm = aStupidModelSwarm;
    return self;
}

/////////////////////////////////
//
// setBugInitialSize
//
///////////////////////////////
- setBugInitialSize: (double) anInitSize
{
    mySize = anInitSize;
    return self;
}

/////////////////////////////////
//
// setBugWorld
//
////////////////////////////////
- setBugWorld: (id <Grid2d>) aWorld
{
   myBugWorld = aWorld;
   return self;
}
	

/////////////////////////////////
//
// setCellWorld
//
////////////////////////////////
- setCellWorld: (id <Grid2d>) aWorld
{
   myCellWorld = aWorld;
   return self;
}
	

/////////////////////////////////////////
//
// setRandPosition
//
///////////////////////////////////////
- setRandPosition
{
    int worldSizeX = [myBugWorld getSizeX];
    int worldSizeY = [myBugWorld getSizeY];

    if(myBugWorld == nil)
    {
       fprintf(stderr, "ERROR: StupidBug >>>> setRandPosition >>>> myWorld is nil\n");
       fflush(0);
       exit(1);
    }    
 
    do 
    {
        myX = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeX - 1];
        myY = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeY - 1];

    } while ([myBugWorld getObjectAtX: myX Y: myY] != nil);

    [myBugWorld putObject: self atX: myX Y: myY];

    return self;
}


////////////////////////////////////
//
// getMySize
//
///////////////////////////////////
- (double) getMySize
{
   return mySize;
}




/////////////////////////////////////////////////////
//
// drawSelfOn
//
// Starting with Version 2, the intensity of red
// is proportional to bug size. We must set a size
// at which redness is maximum
// In the observer swarm, the color map is given
// red shades from 0 to 63.
//
/////////////////////////////////////////////////////
- drawSelfOn: (id <Raster>) aRaster 
{
   int maxDrawSize = 10;
   int drawSizeRatio = (int) (63*(mySize/maxDrawSize));
   Color drawColor = (Color) (drawSizeRatio < 63) ? drawSizeRatio : 63;

   [aRaster drawPointX: myX Y: myY Color: drawColor];
		
   return self;
}

	

////////////////////////////////////
//
// move
//
////////////////////////////////////
- move 
{
   int moveDistance = 4;
   int newX, newY;

   // Test print to make sure bugs are sorted into descending size
   // fprintf(stdout, "bug size %-12f \n", mySize);

   do 
   {
       newX = [self xnorm: myX + [uniformIntRand getIntegerWithMin: -moveDistance withMax: moveDistance]];
       newY = [self ynorm: myY + [uniformIntRand getIntegerWithMin: -moveDistance withMax: moveDistance]];

   } while ([myBugWorld getObjectAtX: newX Y: newY] != nil);
	
   [myBugWorld putObject: nil atX: myX Y: myY];

   myX = newX;
   myY = newY;

   [myBugWorld putObject: self atX: newX Y: newY];

   return self;
}


//
// These are stolen directly from Repast's toroid space
//
- (int) xnorm: (int) x
{
    int worldSizeX = [myBugWorld getSizeX];

    if((x > worldSizeX - 1) || (x < 0)) 
    {
        while(x < 0)
        {
            x += worldSizeX;
        }

        return x % worldSizeX;
    }

    return x;
}

- (int) ynorm: (int) y
{
    int worldSizeY = [myBugWorld getSizeY];

    if((y > worldSizeY - 1) || (y < 0)) 
    {
           while(y < 0) 
           {
               y += worldSizeY;
           }

           return y % worldSizeY;
    }

    return y;
}



////////////////////////////////////
//
// grow
//
///////////////////////////////////
- grow
{
    HabitatCell* myCell = [myCellWorld getObjectAtX: myX Y:myY];
    double bugMaxConsumptionRate;
    double cellFoodAvailable;
    double foodEaten;

    if(myCell == nil)
    {
        fprintf(stdout, "ERROR: StupidBug >>>> grow >>>> myCell is nil\n");
        exit(1);
        fflush(0);
    }
    if(stupidModelSwarm == nil)
    {
        fprintf(stdout, "ERROR: StupidBug >>>> grow >>>> stupidModelSwarm is nil\n");
        exit(1);
        fflush(0);
    }
           
    bugMaxConsumptionRate = [stupidModelSwarm getBugMaxConsumptionRate];
    cellFoodAvailable = [myCell getCellFoodAvailable];
    foodEaten = (cellFoodAvailable < bugMaxConsumptionRate) ? cellFoodAvailable : bugMaxConsumptionRate;

    [myCell setCellFoodAvailable: (cellFoodAvailable - foodEaten)];

    mySize += foodEaten;

    return self;
}

/////////////////////////////////////////
//
// compare
//
// Used by Qsort to sort the bugs by size
//
/////////////////////////////////////////
- (int) compare: (StupidBug *) otherBug
{
    double otherBugSize = [otherBug getMySize];
    
    if(mySize > otherBugSize) 
    {
       return -1;
    }
    else if(mySize < otherBugSize)
    {   
       return 1;
    }
    else
    {
       return 0;
    }
}

@end
