#import "StupidModelSwarm.h"
#import "HabitatCell.h"


#import "StupidBug.h"


@implementation StupidBug

+ create: aZone
{
   StupidBug* stupidBug = [super create: aZone];

   stupidBug->stupidModelSwarm = nil;
   stupidBug->myBugWorld = nil;
   stupidBug->myCellWorld = nil;
   stupidBug->mySize = -1;
   stupidBug->ImAlive = YES;

   stupidBug->neighborCellList = [List create: aZone];

   return stupidBug;
}


////////////////////////////////
//
// setStupidModelSwarm
//
////////////////////////////////
- setStupidModelSwarm: aStupidModelSwarm
{
    stupidModelSwarm = aStupidModelSwarm;
    return self;
}
/////////////////////////////////
//
// setBugInitialSize
//
///////////////////////////////
- setBugInitialSize: (double) anInitSize
{
    mySize = anInitSize;
    return self;
}

/////////////////////////////////
//
// setBugWorld
//
////////////////////////////////
- setBugWorld: (id <Grid2d>) aWorld
{
   myBugWorld = aWorld;
   return self;
}
	

/////////////////////////////////
//
// setCellWorld
//
////////////////////////////////
- setCellWorld: (id <Grid2d>) aWorld
{
   myCellWorld = aWorld;
   return self;
}
	

/////////////////////////////////////////
//
// setRandPosition
//
///////////////////////////////////////
- setRandPosition
{
    int worldSizeX = [myBugWorld getSizeX];
    int worldSizeY = [myBugWorld getSizeY];

    if(myBugWorld == nil)
    {
       fprintf(stderr, "ERROR: StupidBug >>>> setRandPosition >>>> myWorld is nil\n");
       fflush(0);
       exit(1);
    }    
 
    do 
    {
        myX = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeX - 1];
        myY = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeY - 1];

    } while ([myBugWorld getObjectAtX: myX Y: myY] != nil);

    [myBugWorld putObject: self atX: myX Y: myY];

    return self;
}


////////////////////////////////////
//
// getMySize
//
///////////////////////////////////
- (double) getMySize
{
   return mySize;
}




/////////////////////////////////////////////////////
//
// drawSelfOn
//
// Starting with Version 3, the intensity of red
// is proportional to bug size. We must set a size
// at which redness is maximum
// In the observer swarm, the color map is given
// red shades from 0 to 64.
//
/////////////////////////////////////////////////////
- drawSelfOn: (id <Raster>) aRaster 
{
   int maxDrawSize = 10;
   int drawSizeRatio = (int) (63*(mySize/maxDrawSize));
   Color drawColor = (Color) (drawSizeRatio < 63) ? drawSizeRatio : 63;

   [aRaster drawPointX: myX Y: myY Color: drawColor];
		
   return self;
}

	

//////////////////////////////////////////////////////////
//
// move
//
// For version 11, "move" is modified so the bug finds the 
// cell with highest food availability.
//
//////////////////////////////////////////////////////////
- move 
{
    int moveDistance = 4;
    int testMinX, testMaxX, testMinY, testMaxY;
    int testX = 0;
    int testY = 0;
    HabitatCell* bestCell = nil;
    double foodAtBestCell = -1.0;
    id <ListShuffler> cellShuffler = [ListShuffler create: [stupidModelSwarm getModelZone]];

    //
    // Loop to build a list of all the neighboring cells because Swarm
    // does not have a "getNeighbors" method.
    // We could just identify the best within this loop, but in a
    // later version we will need the list.
		    
    // The list of neighbor cells is an instance variable so
    // the bug does not have to re-create it each time.
    // It is instantiated in the constructor method.
    // So- first empty out the list.
		    
    // Version 15: Now the world is not toroidal so we keep
    // bugs from using cells beyond the edge of the space.
    //
    [neighborCellList removeAll];
		
    testMinX = myX - moveDistance;
    if(testMinX < 0) 
    {
       testMinX = 0;
    }
		
    testMaxX = myX + moveDistance;
    if(testMaxX > ([myCellWorld getSizeX] - 1))
    {
         testMaxX = [myCellWorld getSizeX] - 1;
    }		

    testMinY = myY - moveDistance;
    if(testMinY < 0) 
    {
        testMinY = 0;
    }
		
    testMaxY = myY + moveDistance;
    if(testMaxY > ([myCellWorld getSizeY] - 1))
    {
        testMaxY = [myCellWorld getSizeY] - 1;
    }
		    
    for (testX = testMinX; testX <= testMaxX; testX++) 
    {
        for (testY = testMinY; testY <= testMaxY; testY++) 
        {
            //
            // If there is no bug at this location, put the cell
            // at the location on the neighbor list.
            //
            if([myBugWorld getObjectAtX: testX Y: testY] == nil)
            {
                [neighborCellList addLast: [myCellWorld getObjectAtX: testX Y: testY]];
            }
        }
    }

    //
    // Now put our current cell on the list because we're
    // allowed to stay here.
    //
    [neighborCellList addLast: [myCellWorld getObjectAtX :myX Y: myY]];
	
		
    //
    // In version 15 we need to first shuffle the list of cells.
    // Use Swarm's handy list shuffler
    // We'll use its default random distribution
    //
    [cellShuffler shuffleWholeList: neighborCellList];
    
    //
    // Loop through the list of neighbor cells and find
    // the one with highest food availability
    //
    {
        id <ListIndex> cellIndex = [neighborCellList listBegin: scratchZone];
        HabitatCell* nextCell = nil;
        while(([cellIndex getLoc] != End) && ((nextCell = [cellIndex next]) != nil)) 
        {
            double nextFood = [nextCell getCellFoodAvailable];
            if (nextFood > foodAtBestCell) 
            {
                foodAtBestCell = nextFood;
                bestCell = nextCell;
            }
        }
    }

    [myBugWorld putObject: nil atX : myX Y: myY];

    myX = [bestCell getMyX];
    myY = [bestCell getMyY];
    [myBugWorld putObject: self atX: myX Y: myY];

    return self;
}



////////////////////////////////////////////////
//
// foundACellWithX:Y
//
///////////////////////////////////////////////
- (BOOL) foundACellWithX: (int) momsX 
                       Y: (int) momsY 
{
   int testX, testY;
   int testMinX, testMinY, testMaxX, testMaxY;
   int moveDistance = 3;
   HabitatCell* newCell;

   BOOL foundACell = NO;
		
   //fprintf(stdout, "StupidBug >>>> foundACellWithX >>>> BEGIN\n");
   //fflush(0);
		
   //
   // To find a new location, make a list of available cells
   // This approach is copied from "move"
   // Now, in version 15 the space is non-toroidal
   //
   testMinX = momsX - moveDistance;
   if(testMinX < 0) 
   {
       testMinX = 0;
   }

   testMaxX = momsX + moveDistance;
   if(testMaxX > ([myCellWorld getSizeX] - 1))
   {
       testMaxX =  [myCellWorld getSizeX] - 1;
   }		

   testMinY = momsY - moveDistance;
   if(testMinY < 0) 
   {
       testMinY = 0;
   }
	
   testMaxY = momsY + moveDistance;
   if(testMaxY > ([myCellWorld getSizeY] - 1)) 
   {
      testMaxY = [myCellWorld getSizeY] - 1;
   }		

   //
   // To find a new location, make a list of available cells
   // This approach is copied from "move"
   //
   [neighborCellList removeAll];
   for(testX = testMinX; testX <= testMaxX; testX++) 
   {
	for(testY = testMinY; testY <= testMaxY; testY++) 
        {
            //
            // If there is no bug at this location, put the cell
            // at the location on the neighbor list.
            //
            if([myBugWorld getObjectAtX: testX Y: testY] == nil)
            {
                [neighborCellList addLast: [myCellWorld getObjectAtX: testX Y:testY]];
            }
        }
    }
   
   // If there are one or more vacant cells, 
   // the new bug moves to a randomly chosen vacant cell and lives
   //
   if([neighborCellList getCount] > 0) 
   {
        int offset = [uniformIntRand getIntegerWithMin: 0
                                               withMax: [neighborCellList getCount] -1];

	newCell = [neighborCellList atOffset: offset];
			
        myX = [newCell getMyX];
        myY = [newCell getMyY];
        [myBugWorld putObject: self atX: myX Y: myY];

        foundACell = YES;
   }

   //fprintf(stdout, "StupidBug >>>> foundACellWithX >>>> END\n");
   //fflush(0);

   // "foundACell" remains NO if there were no vacant cells
   return foundACell;
}



//
// These are stolen directly from Repast's toroid space
//
- (int) xnorm: (int) x
{
    int worldSizeX = [myBugWorld getSizeX];

    if((x > worldSizeX - 1) || (x < 0)) 
    {
        while(x < 0)
        {
            x += worldSizeX;
        }

        return x % worldSizeX;
    }

    return x;
}

- (int) ynorm: (int) y
{
    int worldSizeY = [myBugWorld getSizeY];

    if((y > worldSizeY - 1) || (y < 0)) 
    {
           while(y < 0) 
           {
               y += worldSizeY;
           }

           return y % worldSizeY;
    }

    return y;
}



////////////////////////////////////
//
// grow
//
///////////////////////////////////
- grow
{
    HabitatCell* myCell = [myCellWorld getObjectAtX: myX Y:myY];
    double bugMaxConsumptionRate;
    double cellFoodAvailable;
    double foodEaten;

    //fprintf(stdout, "StupidBug >>>> grow >>>> BEGIN\n");
    //fflush(0);


    if(myCell == nil)
    {
        fprintf(stdout, "ERROR: StupidBug >>>> grow >>>> myCell is nil\n");
        exit(1);
        fflush(0);
    }
    if(stupidModelSwarm == nil)
    {
        fprintf(stdout, "ERROR: StupidBug >>>> grow >>>> stupidModelSwarm is nil\n");
        exit(1);
        fflush(0);
    }
           
    bugMaxConsumptionRate = [stupidModelSwarm getBugMaxConsumptionRate];
    cellFoodAvailable = [myCell getCellFoodAvailable];
    foodEaten = (cellFoodAvailable < bugMaxConsumptionRate) ? cellFoodAvailable : bugMaxConsumptionRate;

    [myCell setCellFoodAvailable: (cellFoodAvailable - foodEaten)];

    mySize += foodEaten;

    //
    // For version 12, add reproduction to the grow method
    // When we create offspring, we use the same StupidBug create method
    // New bugs can die during their creation (if they do not find
    // a vacant cell) so we need to be careful ordering cleanup
    // of bug lists in the model swarm
    //
   
    if(mySize >= 10.0) 
    {
        int i;
        StupidBug* newBug;
        int parentX = myX;
        int parentY = myY;

        //
        // First make more bugs
        //
        for(i = 1; i <= 5; i++) 
        {
            newBug = [StupidBug create: [stupidModelSwarm getModelZone]];
            [newBug setStupidModelSwarm: stupidModelSwarm];
            [newBug setBugWorld: myBugWorld];
            [newBug setCellWorld: myCellWorld];
            [newBug setBugInitialSize: 0.0];

            if([newBug foundACellWithX: parentX Y: parentY])
            {
                [stupidModelSwarm addBug: newBug];
            }
            else
            {
                 [newBug drop];
                 newBug = nil;
            }
        }
			
        //
        // Then die
        //
        [myBugWorld putObject: nil atX: myX Y: myY];
        [stupidModelSwarm removeBug: self];

        //
        // Because the possibility exists that the we could be put on the
        // stupidModelSwarm's deadBug list twice, once here and once in our die 
        // method, we set the ImAlive to NO.
        //
        ImAlive = NO;
    }

    //fprintf(stdout, "StupidBug >>>> grow >>>> END\n");
    //fflush(0);

    return self;
}
	
//////////////////////////////////////////////////////////////////
//
// die
//
// For version 12, add a method for mortality.
// The survival probability parameter is left in the model
//
//////////////////////////////////////////////////////////////////
- die 
{
   //fprintf(stdout, "StupidBug >>>> die >>>> BEGIN\n");
   //fflush(0);
   
   if(ImAlive)
   {
       if([uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0] > [stupidModelSwarm getBugSurvivalProbability])
       {
           [stupidModelSwarm removeBug: self];
           [myBugWorld putObject: nil atX: myX Y: myY];
           ImAlive = NO;
       }
   }

   //fprintf(stdout, "StupidBug >>>> die >>>> END\n");
   //fflush(0);
   return self;
}

/////////////////////////////////////////
//
// compare
//
/////////////////////////////////////////
- (int) compare: (StupidBug *) otherBug
{
    double otherBugSize = [otherBug getMySize];
    
    if(mySize > otherBugSize) 
    {
       return -1;
    }
    else if(mySize < otherBugSize)
    {   
       return 1;
    }
    else
    {
       return 0;
    }
}


- (void) drop
{
    // First, drop the list object we created
    if(neighborCellList != nil)
    {
        [neighborCellList removeAll];
        [neighborCellList drop];
        neighborCellList = nil;
    }
    // Then drop ourselves by calling our superclass's (SwarmObject) drop
    [super drop];
}

@end
