#import "StupidModelSwarm.h"
#import "HabitatCell.h"
#import "StupidBug.h"


#import "StupidPredator.h"


@implementation StupidPredator

+ create: aZone
{
   StupidPredator* stupidPredator = [super create: aZone];

   stupidPredator->stupidModelSwarm = nil;
   stupidPredator->myBugWorld = nil;
   stupidPredator->myCellWorld = nil;
   stupidPredator->myPredWorld = nil;

   stupidPredator->neighborCellList = [List create: aZone];

   return stupidPredator;
}


////////////////////////////////
//
// setStupidModelSwarm
//
////////////////////////////////
- setStupidModelSwarm: aStupidModelSwarm
{
    stupidModelSwarm = aStupidModelSwarm;
    return self;
}


/////////////////////////////////
//
// setBugWorld
//
////////////////////////////////
- setBugWorld: (id <Grid2d>) aWorld
{
   myBugWorld = aWorld;
   return self;
}
	

/////////////////////////////////
//
// setCellWorld
//
////////////////////////////////
- setCellWorld: (id <Grid2d>) aWorld
{
   myCellWorld = aWorld;
   return self;
}
	

/////////////////////////////////
//
// setPredWorld
//
////////////////////////////////
- setPredWorld: (id <Grid2d>) aWorld
{
   myPredWorld = aWorld;
   return self;
}
	

/////////////////////////////////////////
//
// setRandPosition
//
///////////////////////////////////////
- setRandPosition
{
    int worldSizeX = [myPredWorld getSizeX];
    int worldSizeY = [myPredWorld getSizeY];

    if(myPredWorld == nil)
    {
       fprintf(stderr, "ERROR: StupidPredator >>>> setRandPosition >>>> myWorld is nil\n");
       fflush(0);
       exit(1);
    }    
 
    do 
    {
        myX = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeX - 1];
        myY = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeY - 1];

    } while ([myPredWorld getObjectAtX: myX Y: myY] != nil);

    [myPredWorld putObject: self atX: myX Y: myY];

    return self;
}






/////////////////////////////////////////////////////
//
// drawSelfOn
//
// In the observer swarm, we have set the colormap's
// color # 128 to orange, for predators.
//
/////////////////////////////////////////////////////
- drawSelfOn: (id <Raster>) aRaster 
{
   [aRaster drawPointX: myX Y: myY Color: (Color) 128];
   return self;
}

	


///////////////////
//
// hunt
//
///////////////////
- hunt
{
    int moveDistance = 1;
    int testX, testY; 
    int testMinX, testMaxX, testMinY, testMaxY;
    id <ListShuffler> cellShuffler;
    id <ListIndex> cellIndex;
    HabitatCell* nextCell;
    HabitatCell* newCell;
    StupidBug* preyBug  = nil;
    StupidPredator* anotherPredator  = nil;

    //
    // Loop to build a list of all the neighboring cells because Swarm
    // does not have a "getNeighbors" method.
    //
    
    //
    // The list of neighbor cells is an instance variable so
    // the predator does not have to re-create it each time.
    // It is instantiated in the create method.
    // So- first empty out the list.
    //
    [neighborCellList removeAll];

    testMinX = myX - moveDistance;
    if(testMinX < 0)
    {
        testMinX = 0;
    }		

    testMaxX = myX + moveDistance;
    if(testMaxX > ([myCellWorld getSizeX] - 1))
    {
        testMaxX = [myCellWorld getSizeX] - 1;
    }

    testMinY = myY - moveDistance;
    if(testMinY < 0)
    {
        testMinY = 0;
    }		

    testMaxY = myY + moveDistance;
    if(testMaxY > ([myCellWorld getSizeY] - 1)) 
    {
       testMaxY = [myCellWorld getSizeY] - 1;
    }		


    for(testX = testMinX; testX <= testMaxX; testX++) 
    {
        for(testY = testMinY; testY <= testMaxY; testY++) 
        {
            [neighborCellList addLast: [myCellWorld getObjectAtX: testX Y: testY]];
        }
    }

    //
    // Loop through the list of neighbor cells and see
    // if there are any bugs to eat.
    // We need to first shuffle the list of cells.
    //
    cellShuffler = [ListShuffler create: [stupidModelSwarm getModelZone]];
    [cellShuffler shuffleWholeList: neighborCellList];
    [cellShuffler drop];
		
    cellIndex = [neighborCellList listBegin: scratchZone];
    while(([cellIndex getLoc] != End) && ((nextCell = [cellIndex next]) != nil)) 
    {
        //
        // If there is another predator in the cell, stop hunting
        // and remain at our current location.
        //
        if((anotherPredator = [myPredWorld getObjectAtX: [nextCell getMyX] Y: [nextCell getMyY]]) != nil) 
        {
            if([myPredWorld getObjectAtX: [nextCell getMyX] Y: [nextCell getMyY]] != self) 
            {
                  break;
            }
        }


        //
        // If there's a bug in the cell, kill it and move there
        // Note that bug lists will get screwed up if prey hunting
        // were scheduled before bug actions - bugs killed by predators
        // would still do bug actions.
        //
        if((preyBug = [myBugWorld getObjectAtX: [nextCell getMyX] Y: [nextCell getMyY]]) != nil) 
        {
              [stupidModelSwarm removeBug: preyBug];
              [myBugWorld putObject: nil atX: [nextCell getMyX] Y: [nextCell getMyY]];
              [myPredWorld putObject: nil atX: myX Y: myY];
              myX = [nextCell getMyX];
              myY = [nextCell getMyY];
              [myPredWorld putObject: self atX: myX Y: myY];
				
              //
              // And stop hunting
              //
              break;
         }
			
    } // End of loop through neighbor cells

    [cellIndex drop];

    if((anotherPredator == nil) && (preyBug == nil))
    {
        //
        // If we found no prey, and encountered no other predator (ourselves included),
        // move randomly.   Because the cell list is shuffled, 
        // just move to the first cell on it.
        //
        [myPredWorld putObject: nil atX: myX Y: myY];
        newCell = [neighborCellList getFirst];
        myX = [newCell getMyX];
        myY = [newCell getMyY];

        if([myPredWorld getObjectAtX: myX Y: myY] != nil)
        {
            fprintf(stderr, "ERROR: StupidPredator >>>> hunt >>>> overwriting predator at %d %d\n", myX, myY);
            fflush(0);
            exit(1);
        }

        [myPredWorld putObject: self atX: myX Y: myY];
   }
		    
   return self;
}




- (void) drop
{
    if(neighborCellList != nil)
    {
        [neighborCellList removeAll];
        [neighborCellList drop];
        neighborCellList = nil;
    }

    [super drop];
}

@end
