#import "HabitatCell.h"


#import "StupidBug.h"


@implementation StupidBug

+ create: aZone
{
   StupidBug* stupidBug = [super create: aZone];

   stupidBug->myWorld = nil;
   stupidBug->mySize = -1;
   stupidBug->bugMaxConsumptionRate = -1;

   return stupidBug;
}


/////////////////////////////////
//
// setBugInitialSize
//
///////////////////////////////
- setBugInitialSize: (double) anInitSize
{
    mySize = anInitSize;
    return self;
}

//////////////////////////////////////////////////////////
//
// setBugMaxConsumptionRate
//
//////////////////////////////////////////////////////////
- setBugMaxConsumptionRate: (double) aMaxConsumptionRate
{
    bugMaxConsumptionRate = aMaxConsumptionRate;
    return self;
}

/////////////////////////////////
//
// setWorld
//
////////////////////////////////
- setWorld: (id <Grid2d>) aWorld
{
   myWorld = aWorld;
   return self;
}
	

/////////////////////////////////////////
//
// setRandCell
//
////////////////////////////////////////
- setRandCell
{
    int worldSizeX = [myWorld getSizeX];
    int worldSizeY = [myWorld getSizeY];
    HabitatCell* tempCell;

    if(myWorld == nil)
    {
       fprintf(stderr, "ERROR: StupidBug >>>> setRandPosition >>>> myWorld is nil\n");
       fflush(0);
       exit(1);
    }    
 
    do 
    {
        myX = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeX - 1];
        myY = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeY - 1];
	tempCell = (HabitatCell *) [myWorld getObjectAtX: myX Y: myY];

    } while ([tempCell getMyBug] != nil);

    [tempCell setMyBug: self];

    return self;
}


/////////////////////////////////////////////////////
//
// drawSelfOn
//
// Starting with Version 2, the intensity of red
// is proportional to bug size. We must set a size
// at which redness is maximum
// In the observer swarm, the color map is given
// red shades from 0 to 63.
//
/////////////////////////////////////////////////////
- drawSelfOn: (id <Raster>) aRaster 
{
   int maxDrawSize = 10;
   int drawSizeRatio = (int) (63*(mySize/maxDrawSize));
   Color drawColor = (Color) (drawSizeRatio < 63) ? drawSizeRatio : 63;

   [aRaster drawPointX: myX Y: myY Color: drawColor];
		
   return self;
}

	

////////////////////////////////////
//
// move
//
////////////////////////////////////
- move 
{
   int moveDistance = 4;
   int newX, newY;
   id newCell = nil;
   id oldCell = [myWorld getObjectAtX: myX Y: myY];

   do 
   {
       newX = [self xnorm: myX + [uniformIntRand getIntegerWithMin: -moveDistance withMax: moveDistance]];
       newY = [self ynorm: myY + [uniformIntRand getIntegerWithMin: -moveDistance withMax: moveDistance]];
       newCell = (HabitatCell *) [myWorld getObjectAtX: newX Y: newY];

   } while([newCell getMyBug] != nil);
	
   [oldCell setMyBug: nil];

   myX = newX;
   myY = newY;

   [newCell setMyBug: self];

   return self;
}


//
// These are stolen directly from Repast's toroid space
//
- (int) xnorm: (int) x
{
    int worldSizeX = [myWorld getSizeX];

    if((x > worldSizeX - 1) || (x < 0)) 
    {
        while(x < 0)
        {
            x += worldSizeX;
        }

        return x % worldSizeX;
    }

    return x;
}

- (int) ynorm: (int) y
{
    int worldSizeY = [myWorld getSizeY];

    if((y > worldSizeY - 1) || (y < 0)) 
    {
           while(y < 0) 
           {
               y += worldSizeY;
           }

           return y % worldSizeY;
    }

    return y;
}



////////////////////////////////////
//
// grow
//
///////////////////////////////////
- grow
{
    // Starting in Version 3, growth is limited by the cell's food availability.
    id myCell = [myWorld getObjectAtX: myX Y:myY];
    double cellFoodAvailable = [myCell getCellFoodAvailable];
    double foodEaten = (cellFoodAvailable < bugMaxConsumptionRate) ? cellFoodAvailable : bugMaxConsumptionRate;

    [myCell setCellFoodAvailable: (cellFoodAvailable - foodEaten)];

    mySize += foodEaten;

    return self;
}

@end
