#import "HabitatCell.h"


#import "StupidBug.h"


@implementation StupidBug

+ create: aZone
{
   StupidBug* stupidBug = [super create: aZone];

   stupidBug->myBugWorld = nil;
   stupidBug->myCellWorld = nil;
   stupidBug->mySize = -1;
   stupidBug->bugMaxConsumptionRate = -1;

   return stupidBug;
}




/////////////////////////////////
//
// setBugInitialSize
//
///////////////////////////////
- setBugInitialSize: (double) anInitSize
{
    mySize = anInitSize;
    return self;
}

//////////////////////////////////////////////////////////
//
// setBugMaxConsumptionRate
//
//////////////////////////////////////////////////////////
- setBugMaxConsumptionRate: (double) aMaxConsumptionRate
{
    bugMaxConsumptionRate = aMaxConsumptionRate;
    return self;
}

/////////////////////////////////
//
// setBugWorld
//
////////////////////////////////
- setBugWorld: (id <Grid2d>) aWorld
{
   myBugWorld = aWorld;
   return self;
}
	

/////////////////////////////////
//
// setCellWorld
//
////////////////////////////////
- setCellWorld: (id <Grid2d>) aWorld
{
   myCellWorld = aWorld;
   return self;
}
	

/////////////////////////////////////////
//
// setRandPosition
//
///////////////////////////////////////
- setRandPosition
{
    int worldSizeX = [myBugWorld getSizeX];
    int worldSizeY = [myBugWorld getSizeY];

    if(myBugWorld == nil)
    {
       fprintf(stderr, "ERROR: StupidBug >>>> setRandPosition >>>> myWorld is nil\n");
       fflush(0);
       exit(1);
    }    
 
    do 
    {
        myX = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeX - 1];
        myY = [uniformIntRand getIntegerWithMin: 0 withMax: worldSizeY - 1];

    } while ([myBugWorld getObjectAtX: myX Y: myY] != nil);

    [myBugWorld putObject: self atX: myX Y: myY];

    return self;
}


/////////////////////////////////////////////////////
//
// drawSelfOn
//
// Starting with Version 3, the intensity of red
// is proportional to bug size. We must set a size
// at which redness is maximum
// In the observer swarm, the color map is given
// red shades from 0 to 255.
//
/////////////////////////////////////////////////////
- drawSelfOn: (id <Raster>) aRaster 
{
   int maxDrawSize = 10;
   int drawSizeRatio = (int) (63*(mySize/maxDrawSize));
   Color drawColor = (Color) (drawSizeRatio < 63) ? drawSizeRatio : 63;

   [aRaster drawPointX: myX Y: myY Color: drawColor];
		
   return self;
}

	

////////////////////////////////////
//
// move
//
////////////////////////////////////
- move 
{
   int moveDistance = 4;
   int newX, newY;

   do 
   {
       newX = [self xnorm: myX + [uniformIntRand getIntegerWithMin: -moveDistance withMax: moveDistance]];
       newY = [self ynorm: myY + [uniformIntRand getIntegerWithMin: -moveDistance withMax: moveDistance]];

   } while ([myBugWorld getObjectAtX: newX Y: newY] != nil);
	
   [myBugWorld putObject: nil atX: myX Y: myY];

   myX = newX;
   myY = newY;

   [myBugWorld putObject: self atX: newX Y: newY];

   return self;
}


//
// These are stolen directly from Repast's toroid space
//
- (int) xnorm: (int) x
{
    int worldSizeX = [myBugWorld getSizeX];

    if((x > worldSizeX - 1) || (x < 0)) 
    {
        while(x < 0)
        {
            x += worldSizeX;
        }

        return x % worldSizeX;
    }

    return x;
}

- (int) ynorm: (int) y
{
    int worldSizeY = [myBugWorld getSizeY];

    if((y > worldSizeY - 1) || (y < 0)) 
    {
           while(y < 0) 
           {
               y += worldSizeY;
           }

           return y % worldSizeY;
    }

    return y;
}



////////////////////////////////////
//
// grow
//
///////////////////////////////////
- grow
{
    HabitatCell* myCell = (HabitatCell *) [myCellWorld getObjectAtX: myX Y:myY];
    double cellFoodAvailable = [myCell getCellFoodAvailable];
    double foodEaten = (cellFoodAvailable < bugMaxConsumptionRate) ? cellFoodAvailable : bugMaxConsumptionRate;

    [myCell setCellFoodAvailable: (cellFoodAvailable - foodEaten)];

    mySize += foodEaten;

    return self;
}

@end
