#import "StupidModelSwarm.h"


@implementation StupidModelSwarm

+ create: (id) aZone 
{
  StupidModelSwarm* obj = [super create: aZone];

  obj->cellInitialFoodAvailability = 0.0;
  obj->cellMaxFoodProductionRate = 0.01;
  obj->bugInitSize = 1.0;
  obj->bugMaxConsumptionRate = 1.0;

  return obj;
}



////////////////////////////////////
//
// buildObjects
//
///////////////////////////////////
- buildObjects 
{
  int i;
  int j;

  //
  // allow our parent class to build anything.
  //
  [super buildObjects];

  //
  // Create a modelZone in which to manage our objects
  //
  modelZone = [Zone create: [self getZone]];

  //
  // Now set up the grid used to represent agent position
  //
  cellWorld = [Grid2d create: modelZone setSizeX: 100 Y: 100];


  //
  // Create a list for the cells
  //
  cellList = [List create: modelZone]; 
  
  //
  // Create the cells, so the bugs can move into them
  //
  for(i=0; i<100; i++)
  {
      for(j=0; j<100; j++)
      {
          HabitatCell* newCell = [HabitatCell create: modelZone];
          [newCell setMyX: i];
          [newCell setMyY: j];
          [newCell setCellFoodAvailable: cellInitialFoodAvailability];
          [newCell setCellMaxFoodProdRate: cellMaxFoodProductionRate];
          [cellWorld putObject: newCell atX: i Y: j];
          [cellList addLast: newCell];
      }
  }

  // Now set up the separate grid used for bug locations
  bugWorld = [Grid2d create: modelZone setSizeX: 100 Y: 100];

  //
  // Create a list of the stupidBugs
  //
  bugList = [List create: modelZone];
  
  //
  // Make some agents
  // The bugs put themselves on the space.
  //
  for(i = 0; i < 100; i++) 
  {
      StupidBug* stupidBug = nil;

      stupidBug = [StupidBug create: modelZone];
      [stupidBug setBugWorld: bugWorld];
      [stupidBug setCellWorld: cellWorld];
      [stupidBug setRandPosition];
      [stupidBug setBugInitialSize: bugInitSize];
      [stupidBug setBugMaxConsumptionRate: bugMaxConsumptionRate];
 
      [bugList addLast: stupidBug];
  }

  return self;
}
	

 

/////////////////////////////////
//
// buildActions
//
/////////////////////////////////
- buildActions 
{
  //fprintf(stdout, "StupidModelSwarm >>>> buildActions >>>> BEGIN\n");

  [super buildActions];

  //
  // Now the first actions are for the habitat cells
  //
  cellActions = [ActionGroup create: modelZone];
  [cellActions createActionForEach: cellList message: M(growFood)];


  //
  // Create an action group for agent actions
  //
  bugActions = [ActionGroup create: modelZone];

  //
  // Next, create the "move" and "grow" actions for bugs
  // Include the StupidBug interface (i.e. StupidBug.h) in StupidModelSwarm.h
  //
  [bugActions createActionForEach: bugList message: M(move)];
  [bugActions createActionForEach: bugList message: M(grow)];

  //
  // Then we create a schedule and put the 
  // action group(s) on it. 	      
  //
  modelSchedule = [Schedule createBegin: modelZone];
  [modelSchedule setRepeatInterval: 1];
  modelSchedule = [modelSchedule createEnd];
  [modelSchedule at: 0 createAction: cellActions];
  [modelSchedule at: 0 createAction: bugActions];

  //fprintf(stdout, "StupidModelSwarm >>>> buildActions >>>> END\n");

  return self;
}




/////////////////////////////////////////////
//
// activateIn
//
////////////////////////////////////////////
- activateIn:  swarmContext 
{
  //
  // First, activate ourselves via the superclass
  // activateIn: method.  Just pass along the context: the
  // activity library does the right thing.
  //
  [super activateIn: swarmContext];

  //
  // Now activate our own schedule.
  //
  [modelSchedule activateIn: self];

  //
  // Finally, return our activity.
  //
  return [self getSwarmActivity];
}



/////////////////////////////
//
// getCellWorld
//
////////////////////////////
- (id <Grid2d>) getCellWorld
{
   return cellWorld;
}

/////////////////////////////
//
// getBugWorld
//
////////////////////////////
- (id <Grid2d>) getBugWorld
{
   return bugWorld;
}


///////////////////////////////
//
// getBugList
//
//////////////////////////////
- (id <List>) getBugList
{
    return bugList;
}


///////////////////////////////
//
// getCellList
//
//////////////////////////////
- (id <List>) getCellList
{
    return cellList;
}

@end
