#import "StupidObserverSwarm.h"
#import <simtoolsgui.h>

@implementation StupidObserverSwarm

+ createBegin: aZone
{
  StupidObserverSwarm * obj;
  
  obj = [super createBegin: aZone];

  obj->observerZone = [Zone create: aZone];

  return obj;
}

- createEnd
{
  return [super createEnd];
}

- buildObjects
{
  int i;
  double colorScaler;

  [super buildObjects];

  //
  // First, we create the model that we're actually observing. The
  // model is a subswarm of the observer. 
  //
  stupidModelSwarm = [StupidModelSwarm create: self];

  //
  // This macro creates a probe display for the given object
  // The probes are defined in the StupidModelSwarm's create
  // method.
  //
  CREATE_PROBE_DISPLAY (stupidModelSwarm);
  



  //
  // Instruct the control panel to wait for a button event: we
  // halt here until someone hits a control panel button so the
  // user can get a chance to fill in parameters before the
  // simulation runs
  //
  [controlPanel setStateStopped];

  //
  // Now, when the control panel is started, 
  // the model swarm builds its objects.
  //
  [stupidModelSwarm buildObjects];

  //
  // We need a color map to provide colors for bugs, etc.
  //
  colormap = [Colormap create: observerZone];

  //
  // Starting with version 2, we need a gradient of increasing redness
  //
  for(i = 0; i < 64; i++) 
  {
      colorScaler = ((double) i) / 63;

      [colormap setColor: (Color) i 
                   ToRed: 1  
                   Green: 1.0 - colorScaler 
                    Blue: 1.0 - colorScaler];
  }



  //
  // Next, create a GUI window for display, set its size, zoom
  // factor, title.
  //
  worldRaster = [ZoomRaster createBegin: observerZone];
  SET_WINDOW_GEOMETRY_RECORD_NAME (worldRaster);
  worldRaster = [worldRaster createEnd];
  [worldRaster setColormap: colormap];
  [worldRaster setZoomFactor: 4];
  [worldRaster setWidth: [[stupidModelSwarm getCellWorld] getSizeX] 
                 Height: [[stupidModelSwarm getCellWorld] getSizeY]];
  [worldRaster setWindowTitle: "Stupid World"];

  //
  // draw the window.
  //
  [worldRaster pack];
	        
  // In version 4, we add separate Object2dDisplays for
  // each of the (now) two grid2D objects, which contain cells and bugs;
  cellDisplay = [Object2dDisplay createBegin: observerZone];
  [cellDisplay setDisplayWidget: worldRaster];
  [cellDisplay setDiscrete2dToDisplay: [stupidModelSwarm getCellWorld]];
  [cellDisplay setDisplayMessage: M(drawSelfOn:)];   // draw method
  cellDisplay = [cellDisplay createEnd];
  
  //
  // And also create an Object2dDisplay: this object draws
  // stupidbugs on the worldRaster widget for us
  //
  bugDisplay = [Object2dDisplay createBegin: observerZone];
  [bugDisplay setDisplayWidget: worldRaster];
  [bugDisplay setDiscrete2dToDisplay: [stupidModelSwarm getBugWorld]];
  [bugDisplay setDisplayMessage: M(drawSelfOn:)];   // draw method
  bugDisplay = [bugDisplay createEnd];

  [worldRaster setButton: ButtonLeft
                  Client: cellDisplay 
                 Message: M(makeProbeAtX:Y:)];

  [worldRaster setButton: ButtonRight
                  Client: bugDisplay 
                 Message: M(makeProbeAtX:Y:)];

		    
  // In version 6, we add a histogram of bug sizes

  bugSizeHistogram = [EZBin createBegin: observerZone];
  SET_WINDOW_GEOMETRY_RECORD_NAME (bugSizeHistogram);
  [bugSizeHistogram setTitle: "Bug Sizes"];
  [bugSizeHistogram setAxisLabelsX: "Bug Size" Y: "Number of Bugs"];
  [bugSizeHistogram setBinCount: 10];
  [bugSizeHistogram setLowerBound: 0];
  [bugSizeHistogram setUpperBound: 100];
  [bugSizeHistogram setCollection: [stupidModelSwarm getBugList]];
  [bugSizeHistogram setProbedSelector: M(getMySize)];
  bugSizeHistogram = [bugSizeHistogram createEnd];
  
  

  return self;
}  

- buildActions
{
  //System.out.println("StupidObserverSwarm >>>> buildActions >>>> BEGIN");

  [super buildActions];
  
  //
  // First, let our model swarm build its own schedule.
  //
  [stupidModelSwarm buildActions];

  //
  // Create an ActionGroup for display
  //
  displayActions = [ActionGroup create: self];

  //
  // Add the methods to the ActionGroup to draw the display
  //
  [displayActions createActionTo: probeDisplayManager message: M(update)];
  [displayActions createActionTo: self message: M(update)];

  //
  // Finally, create an action to update for the whole user
  // interface code.  
  //
  [displayActions createActionTo: actionCache message: M(doTkEvents)];

  // In version 7, we:
  // Create an ActionGroup for stopping the model
  //
  checkStopActions = [ActionGroup create: self];
  [checkStopActions createActionTo: self message: M(checkToStop)];

  //
  // Create the Observer Swarm's display schedule.
  // Use a display update frequency of 1.
  //
  displaySchedule = [Schedule createBegin: self];
  [displaySchedule setRepeatInterval: 1];
  displaySchedule = [displaySchedule createEnd];

  //
  // insert the ActionGroup on the Schedule
  //
  [displaySchedule at: 0 createAction: displayActions];
  [displaySchedule at: 0 createAction: checkStopActions];

  //fprintf(stdout, "StupidObserverSwarm >>>> buildActions >>>> END\n");

  return self;
}  




/*
    activateIn is called by Main; it puts the schedules of the
    Observer and Model swarms together in one overall schedule.
    A swarm's "activateIn(aSwarm)" method means "execute my schedule in
    the schedule of aSwarm". "aSwarm" is referred to as the "context". 
*/
- activateIn: swarmContext
{
  //
  // First, activate ourselves (just pass along the context).
  //
  [super activateIn: swarmContext];

  /* Activate the model swarm in ourselves. This makes the model 
     swarm a subswarm of the observer swarm -- and executes its
     schedule before the observer's each time step.
  */
  [stupidModelSwarm activateIn: self];

  //
  // Now activate our schedule in ourselves. This arranges for
  // the execution of the schedule we built.
  //
  [displaySchedule activateIn: self];
  
  //
  // Activate returns the swarm activity - the thing that's ready to run.
  //
  return [self getSwarmActivity];
}

//////////////////////////////////////
//
// update
//
//////////////////////////////////////
- update
{
   //fprintf(stdout, "StupidObserverSwarm >>>> update >>>> BEGIN\n");
 
 if(worldRaster != nil)
 {
   [worldRaster erase];
   [bugDisplay display];
   [worldRaster drawSelf];
 }

 if(bugSizeHistogram != nil) 
 {
    [bugSizeHistogram reset];
    [bugSizeHistogram update];
    [bugSizeHistogram output];
 }
 
   //fprintf(stdout, "StupidObserverSwarm >>>> update >>>> END\n");

   return self;
}




/////////////////////////////////
//
// checkToStop
//
/////////////////////////////////
- checkToStop
{
   if ([stupidModelSwarm saysToStop]) 
   {
      [controlPanel setStateQuit];
   }

   //fprintf(stdout, "Observer checking to stop\n");
   //fflush(0);

   return self;
}

@end
